
/******************************************************************************
* MODULE     : tree_label.hh
* DESCRIPTION: labels of trees
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef TREE_LABEL_H
#define TREE_LABEL_H
#include "string.hh"

/******************************************************************************
* Primitive tree labels
******************************************************************************/

enum tree_label {
  STRING, UNKNOWN, UNINIT, ERROR, RAW_DATA,
  DOCUMENT, PARAGRAPH, SURROUND, CONCAT, FORMAT,
  HSPACE, VSPACE_BEFORE, VSPACE_AFTER, SPACE,
  HTAB, SPLIT, MOVE, RESIZE, REPEAT, FLOAT,
  DECORATE_ATOMS, DECORATE_LINES, DECORATE_PAGES, DECORATED_BOX, // 25

  GROUP, LEFT, MIDDLE, RIGHT, BIG,
  LEFT_PRIME, RIGHT_PRIME, BELOW, ABOVE,
  LEFT_SUB, LEFT_SUP, RIGHT_SUB, RIGHT_SUP,
  FRAC, SQRT, WIDE, WIDE_UNDER, NEG, TREE,
  OLD_MATRIX, OLD_TABLE, OLD_MOSAIC, OLD_MOSAIC_ITEM, // 23

  TABLE_FORMAT, TABLE_WITH, CELL_WITH, TABLE_MARKER,
  TABLE, ROW, CELL, SUB_TABLE, // 8

  ASSIGN, WITH, SET, RESET,
  EXPAND, VAR_EXPAND, HIDE_EXPAND,
  APPLY, BEGIN, END, INCLUDE,
  MACRO, FUNCTION, ENVIRONMENT,
  EVAL, PROVIDES, VALUE, ARGUMENT,
  BACKUP, QUOTE, DELAY, HOLD, RELEASE, // 23

  OR, XOR, AND, NOT,
  PLUS, MINUS, TIMES, OVER, DIVIDE, MODULO,
  MERGE, LENGTH, RANGE, NUMBER, DATE, TRANSLATE, FIND_FILE,
  IS_TUPLE, LOOK_UP,
  EQUAL, UNEQUAL, LESS, LESSEQ, GREATER, GREATEREQ,
  IF, VAR_IF, CASE, WHILE,
  EXTERN, AUTHORIZE, // 31

  INACTIVE, SYMBOL, LATEX, HYBRID,
  TUPLE, COLLECTION, ASSOCIATE,
  LABEL, REFERENCE, PAGEREF, WRITE,
  SPECIFIC, HYPERLINK, ACTION,
  TAG, MEANING, // 16

  GRAPHICS, SUPERPOSE, TEXT_AT, POINT,
  LINE, CLINE, SPLINE, VAR_SPLINE, CSPLINE,
  FILL, POSTSCRIPT // 11
};

inline tree_label SUB (bool right) { return right? RIGHT_SUB: LEFT_SUB; }
inline tree_label SUP (bool right) { return right? RIGHT_SUP: LEFT_SUP; }

/******************************************************************************
* Format strings
******************************************************************************/

extern string WITH_LIMITS;
extern string LINE_BREAK;
extern string NEW_LINE;
extern string LINE_SEP;
extern string NEXT_LINE;
extern string NO_BREAK;
extern string YES_FIRST_INDENT;
extern string NO_FIRST_INDENT;
extern string YES_FIRST_INDENT_AFTER;
extern string NO_FIRST_INDENT_AFTER;
extern string PAGE_BREAK_BEFORE;
extern string PAGE_BREAK;
extern string NO_PAGE_BREAK_BEFORE;
extern string NO_PAGE_BREAK_AFTER;
extern string NEW_PAGE_BEFORE;
extern string NEW_PAGE;
extern string NEW_DOUBLE_PAGE_BEFORE;
extern string NEW_DOUBLE_PAGE;

/******************************************************************************
* Tree label properties
******************************************************************************/

#define CONSTRUCTOR_NR 139
extern string CONSTRUCTOR_NAME       [CONSTRUCTOR_NR];
extern int    CONSTRUCTOR_ARITY      [CONSTRUCTOR_NR];
extern int    CONSTRUCTOR_PROPERTIES [CONSTRUCTOR_NR];

#define ACCESSIBLE_MASK           15
#define NOT_ACCESSIBLE             0
#define ACCESSIBLE                 1
#define FIRST_ACCESSIBLE           2
#define LAST_ACCESSIBLE            3
#define TAIL_ACCESSIBLE            4
#define TABLE_ACCESSIBLE           5
#define HIDE_EXPAND_ACCESSIBLE     6
#define BORDER_ACCESSIBLE_MASK    16
#define BORDER_NOT_ACCESSIBLE     16
#define DYNAMIC_MASK              32
#define DYNAMIC                   32

#define ACCESSIBLE_EXCEPT_BORDER (ACCESSIBLE+BORDER_NOT_ACCESSIBLE)
#define ONLY_LAST_ACCESSIBLE (LAST_ACCESSIBLE+BORDER_NOT_ACCESSIBLE)

#include "hashmap.hh"
template<class T, class U> class hashmap;
extern hashmap<string,int> CONSTRUCTOR_CODE;

void initialize_data ();

#endif // defined TREE_LABEL_H
