
/******************************************************************************
* MODULE     : tohtml.cc
* DESCRIPTION: conversion of TeXmacs trees into Html strings
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "convert.hh"
#include "sys_utils.hh"
#include "scheme.hh"

static tree
substitute_bad_chars (tree t) {
  if (is_atomic (t)) {
    string s= t->label, r;
    int i, n= N(s);
    for (i=0; i<n; i++)
      switch ((int) s[i]) {
      case 0: r << "`"; break;
      case 1: r << "'"; break;
      case 2: r << "^"; break;
      case 3: r << "~"; break;
      case 4: r << "\""; break;
      default: r << s[i];
      }
    return r;
  }
  else {
    int i, n= N(t);
    tree r (t, n);
    for (i=0; i<n; i++)
      r[i]= substitute_bad_chars (t[i]);
    return r;
  }
}

static string
tree_to_html (tree t, tree style, string lan, bool document_flag) {
  if (is_atomic (style)) style= tree (TUPLE, style);
  t= substitute_bad_chars (t);
  if (document_flag) {
    t= tree (EXPAND, "!file", t);
    string tmpath= get_env ("TEXMACS_PATH");
    while ((N(tmpath)>0) && (tmpath [N(tmpath) - 1] == '/'))
      tmpath= tmpath (0, N(tmpath)-1);
    t << style << lan << tmpath;
  }
  return as_string (call ("tmhtml-produce", tree_to_object (t)));
}

string
tree_to_html (tree t, string mode) {
  return tree_to_html (t, "generic", "english", false);
}

string
tree_to_html_document (tree t, tree style, string lan) {
  return tree_to_html (t, style, lan, true);
}
